# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class PublicHousingOffersController < Hermes::Api::Dmo::ApplicationController
        skip_before_filter :verify_authenticity_token, :set_user, :allow_true_user, :check_if_log_as
        skip_after_action :log_action

        def index
          acceptable_params *Hermes::Api::Dmo::HousingOffersController::ACCEPTABLE_PARAMS
          regex_params *Hermes::Api::Dmo::HousingOffersController::REGEX_PARAMS
          valid_operators *Hermes::Api::Dmo::HousingOffersController::VALID_OPERATORS

          @query_precision = { published: true, housing_channel_keys: ["dc"], active_reservations: [] }
          aggregation = index_aggregation(target_model: Hermes::HousingOffer, no_acl: true)
          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1

          paginate(
            aggregation,
            each_serializer: Hermes::PublicHousingOfferSerializer,
            per_page: per_page,
            page: page
          )
        end

        def set_instance
          @instance = Hermes::HousingOffer.find_by(published: true, id: params[:id])
        end

        %w[create update destroy].each do |cud_method_name|
          define_method(cud_method_name) do
            render_json_error :unprocessable_entity, :not_implemented
          end
        end
      end
    end
  end
end
