# frozen_string_literal: true

module Hermes
  # HousingRequest serializer
  class HousingRequestInitializerSerializer < Hermes::ApplicationSerializer
    cache key: proc {"#{object.cache_key}"}, expires_in: 1.minute
    type 'housing_request_initializers'

    attribute :unique_number
    attribute :pin_code_created_at
    attribute :email
    attribute :done
    attribute :expired
    attribute :pin_code

    def is_admin
      !((scope.auth_roles || []) & ['superadministrateur', 'collaborateur offre logement']).empty?
    end 

    def pin_code
      is_admin ? object.pin_code : "unauthorized to see"
    end 

    def expired
      !!(object.pin_code_created_at && object.pin_code_created_at < DateTime.now - 24.hours)
    end 

    def email 
      is_admin ? object.email : object.hidden_email
    end 
  end
end
