# frozen_string_literal: true

require 'faker'

FactoryBot.define do
  factory :housing_offer, class: Hermes::HousingOffer do

    trait :with_lessor do
      association :lessor, factory: :lessor
    end

    after(:build) do |hous_off|
      if hous_off.class.name == 'Hermes::HousingOffer'
        hous_off.class.skip_callback(:create, :after, :call_geocode, raise: false)
      end
    end
  end

  factory :published_offer, parent: :housing_offer do
    department { '34' }
    date_publication_start { Time.new(2020, 3, 15, 0, 0, 0) }
    offer_status { 'PUBLIE' }
    max_applicants { 3 }

    trait :with_applications do
      transient do
        applications_count { 2 }
        housing_request { nil }
      end

      after :create do |offer, evaluator|
        request = evaluator.housing_request.presence || create(:active_request)
        evaluator.applications_count.times do
          create(:housing_application, housing_offer: offer, housing_request: request)
        end
      end
    end
  end
end
