require 'rails_helper'
require 'vcr_config'

RSpec.describe Hermes::RequiredDocument, type: :model do
  let!(:user1) {create(:user)}
  let!(:user2) {create(:user)}
  let!(:entity1) {create(:entity, siret: '00000000000001' )}
  let!(:applicant1) {
    create(
      :applicant,
      user: user1,
      monthly_charges: {"PAV" => 200},
      nationality: "1",
      user_id: user1.id,
      first_name: "robin",
      last_name: "caramaschi",
      birth_date: Date.current,
      main_employer_siret: entity1.siret,
      amount_year_minus_two: 20000,
    )
  }
  
  it 'has a valid factory' do
    expect(FactoryBot.build(:housing_request_folder)).to be_valid
  end


  let!(:housing_request1) do
    create(
      :housing_request,
      is_active: true,
      main_applicant: applicant1,
      user_id: user1.id,
      entity_status: 'EN COURS DE SAISIE',
      form_status: 'complete',
      seeked_departments: ['21']
    )
  end
  let!(:nud1) do
    create(
      :nud,
      housing_request: housing_request1,
      department: '12',
      creation_date: '2016-23-14T14:53:22',
      unique_number: "A1253EF84F3"
    )
  end
  describe "les droits sont transmis" do
    it "is good " do
      hrf = housing_request1.reload.housing_request_folder
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      expect(housing_request1.user_id).to eq(user1.id)
      expect(hrf.user_id).to eq(user1.id)
      expect(required_document.user_id).to eq(user1.id)
    end
  end
  describe "upload file create it in sne" do
    
    before :each do
      entity1.auto_request_validation = true
      entity1.save
      hrf = housing_request1.reload.housing_request_folder
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      file = File.open([Hermes::Engine.root, %w[spec xml_bouchon carte-identite.jpg]].flatten.join('/'))
      blob = file.read
      required_document.set_blobs([{filename: "carte-identite-recto.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
      required_document.save
      housing_request1.housing_request_folder.required_documents[2].update(status: "Pièce validée", validity_limit: Date.current + 5.years)
    end
    it "status are modified correctly" do
      hrf = housing_request1.reload.housing_request_folder
      hrf.reload
      expect(housing_request1.reload.request_status).to eq("EN ATTENTE DE PIECES")
      hrf.start_treatment
      hrf.save
      hrf.reload
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      required_document.status = "Pièce validée"
      required_document.validity_limit = Date.current + 5.years
      required_document.save
      expect(housing_request1.reload.request_status).to eq("ELIGIBLE")
      expect(housing_request1.reload.entity_status).to eq("EN COURS")
    end
    
    it "go found the goods documents in the sne" do
      expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
      
      rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
      res = []
      
      doc = Doc.find(rd.doc_id)
      File.open([Hermes::Engine.root, %w[spec xml_bouchon], "0retourFile.pdf"].flatten.join('/').to_s, 'wb') do |f|
        f.write doc.file.data
      end
      res.push({
                 filename: doc.title,
                 url: doc.file_md5 && "BASEURL/send/#{doc.file_md5}/#{doc.clean_name}"
               })
      
      
      expect(res.length).to eq(1)
      expect(res[0][:filename]).not_to be(nil)
      # expect(rd.reload.last_xml_validation_report).to eq("[]")
      # expect(rd.reload.sne_piece_id).to eq("16768")
      VCR.use_cassette("send_file_to_ged", :match_requests_on => [:method, :uri, :path]) do
        document = Doc.find(rd.doc_id)
        responses = rd.post_to_ged(document)
        expect(responses["data"]["id"]).not_to be(nil)
        data = responses["data"]
        rd.ged_doc_id = data["id"]
        rd.file_url = data["attributes"]["file_url"]
        VCR.use_cassette("read_file_to_ged", :match_requests_on => [:method, :uri, :path]) do
          expect(rd.get_data_from_ged("5e5cd6b677d81d6356ee7edb")["data"]["attributes"]["pages_urls"]).not_to be(nil)
        end
        VCR.use_cassette("delete_file_to_ged", :match_requests_on => [:method, :uri, :path]) do
          #expect(rd.delete_to_ged(rd.ged_doc_id )).to eq("204 No Content")
        end
      end
    end
  end
  describe "kyc_test" do
    before :each do
      hrf = housing_request1.housing_request_folder
      minus_two = hrf.required_documents.detect { |rd| rd.type == "Avis d'imposition 2020 sur les revenus de l'année 2019" }
      hrf.update(
        required_documents: hrf.required_documents - [minus_two]
      )
      minus_two.delete
    end 
    describe "wrong_document" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        required_document.start_treatment("wrong_document")
        expect(required_document.reload.status).to eq("Pièce invalide")
        expect(required_document.reload.validation_errors).to eq([{"message"=>"No MRZ found", "status"=>"KO", "status_code"=>400}])
      end

    end
    describe "identity_recto" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon carte-identite.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "carte-identite-recto.jpg", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
        
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        required_document.start_treatment("identity_recto")
        expect(required_document.reload.validation_errors.length).to eq(3)
      end
    end 
    describe "sejour" do
      before :each do
        applicant1.nationality = "3"
        applicant1.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
       
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon sejour.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "sejour.jpg", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
        
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        required_document.start_treatment("titresejour")
        expect(required_document.reload.validation_errors.length).to eq(3)
      end
    end
    describe "passportspain" do
      before :each do
        applicant1.nationality = "2"
        applicant1.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
    
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon passportspain.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "passportspain.jpg", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
    
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        required_document.start_treatment("passportspain")
        expect(required_document.reload.validation_errors.length).to eq(3)
     #  ap required_document.reload.metas
        
      end
    end
    describe "identity_rectoverso" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon recto_ci.png]].flatten.join('/'))
        blob = file.read
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon verso_ci.png]].flatten.join('/'))
        blob2 = file.read
        required_document.set_blobs([{filename: "carte-identite-recto.png", blob: blob},
                                     {filename: "carte-identite-verso.png", blob: blob2}])
        required_document.save
      end
      it "status are modified correctly" do
    
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        required_document.start_treatment("identity_recto_verso")
        expect(required_document.reload.validation_errors.length).to eq(3)
      end
    end
   
    describe "salaire2" do
      before :each do
        applicant1.nationality = nil
        applicant1.monthly_incomes = {"SAL" => 2000}
        applicant1.work_contract_type = "CDI"
        applicant1.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon 5eebce873db59f1c77dfb225.pdf]].flatten.join('/'))
        blob = file.read
    
        required_document.set_blobs([{filename: "salaire2.pdf", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
    
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        required_document.start_treatment("salaire2")
        expect(required_document.reload.validation_errors.length).to eq(0)

      end
    end
    describe "pieces en ou" do
      before :each do
        applicant1.nationality = nil
        applicant1.monthly_charges = nil
        applicant1.save
        housing_request1.housing_type_list = ["HLM"]
        housing_request1.save
       
        housing_request1.housing_request_folder.required_documents[0].update(status: "Pièce validée", validity_limit: Date.current + 5.years)

        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon 5eebce873db59f1c77dfb225.pdf]].flatten.join('/'))
        blob = file.read
    
        required_document.set_blobs([{filename: "salaire2.pdf", blob: blob}])
        required_document.save

        required_document = housing_request1.reload.housing_request_folder.required_documents[2]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon 5eebce873db59f1c77dfb225.pdf]].flatten.join('/'))
        blob = file.read

        required_document.set_blobs([{filename: "salaire2.pdf", blob: blob}])
        required_document.save

      end
      it "status are modified correctly" do
        expect(housing_request1.reload.housing_request_folder.application_status).to eq "Dossier validé"
      end
    end
    describe "salaire" do
      before :each do
        applicant1.nationality = nil
        applicant1.monthly_incomes = {"SAL" => 2000}
        applicant1.work_contract_type = "CDI"
        applicant1.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon salaire.pdf]].flatten.join('/'))
        blob = file.read
       
        required_document.set_blobs([{filename: "salaire.pdf", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
    
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        required_document.start_treatment("salaire")
        expect(required_document.reload.validation_errors.length).to eq(0)
      end
    end

    describe "impots" do
      before :each do
        applicant1.nationality = nil
        applicant1.amount_year_minus_two = 4000
        applicant1.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon impo6.pdf]].flatten.join('/'))
        blob = file.read
       
        required_document.set_blobs([{filename: "impo8.pdf", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
    
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        required_document.start_treatment("impo8")
        expect(required_document.reload.validation_errors.length).to eq(0)
      end
    end
    describe "impots" do
      before :each do
        applicant1.nationality = nil
        applicant1.amount_year_minus_one = 4000
        applicant1.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon impo6.pdf]].flatten.join('/'))
        blob = file.read
    
        required_document.set_blobs([{filename: "impo8.pdf", blob: blob}])
        required_document.save
      end
      it "status are modified correctly" do
    
        required_document = housing_request1.reload.housing_request_folder.required_documents[1]
        required_document.start_treatment("impo8")
        expect(required_document.reload.validation_errors.length).to eq(0)
      end

    end


  end
  describe "upload file edit it in sne" do
    
    before :each do
      path = [Hermes::Engine.root, %w[spec xml_bouchon], "0_blob_bouchon"].flatten.join('/').to_s
      pn = Pathname.new(path)
      return {err: 'unknown file'} unless pn.exist?
      blob_bouchon = File.read(pn)
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      required_document.set_blobs([{filename: "superPad1.pdf", blob: blob_bouchon}])
      required_document.save
      
      required_document.reload
      return {err: 'unknown file'} unless pn.exist?
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      
      required_document.set_blobs([{filename: "superPadUpdate.pdf", blob: blob_bouchon}])
      required_document.save
    end
    
    it "go found the goods documents in the sne" do
      expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
      rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
      res = []
      
      doc = Doc.find(rd.doc_id)
      res.push({
                 filename: doc.title,
                 url: doc.file_md5 && "BASEURL/send/#{doc.file_md5}/#{doc.clean_name}"
               })
      
      
      expect(res.length).to eq(1)
      expect(res[0][:filename]).not_to be(nil)
      #expect(rd.reload.last_xml_validation_report).to eq("[]")
    end
  end
  
  describe "delete file delete it in sne" do
    
    before :each do
      path = [Hermes::Engine.root, %w[spec xml_bouchon], "0_blob_bouchon"].flatten.join('/').to_s
      pn = Pathname.new(path)
      return {err: 'unknown file'} unless pn.exist?
      blob_bouchon = File.read(pn)
      required_document = housing_request1.reload.housing_request_folder.required_documents[0]
      required_document.set_blobs([{filename: "superPad1.pdf", blob: blob_bouchon}])
      required_document.save
      
      required_document.reload
      required_document.destroy
    end
    
    it "go found the goods documents in the sne" do
      expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
      rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
      expect(rd).to eq(nil)
    end
  end
  
  describe "status" do
    before :each do 
      housing_request1.housing_request_folder.required_documents[2].update(status: "Pièce validée", validity_limit: Date.current + 5.years)
    end 

    describe "status initialisation" do
      it "has status piece manquantes" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        expect(rd.status).to eq("Pièce manquante")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Dossier incomplet")
      end
    end
    describe "ajout de piece" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
      end
      it "has status kyc en cours" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        #  expect(rd.status).to eq("Contrôle en cours")
        #  expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier complet en cours de traitement")
        #  expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Dossier complet en cours de traitement")
      end
    end
    describe "ajout de piece validé kyc" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
        required_document.reload
        required_document.validation_errors = []
        required_document.validity_limit = DateTime.current + 3.months
        
        required_document.save
      end
      it "has status kyc en cours" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        expect(rd.status).to eq("A contrôler")
        expect(housing_request1.reload.housing_request_folder.nud_status).to eq("Dossier validé")
        expect(housing_request1.reload.request_status).to eq("ELIGIBLE")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Dossier validé")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        #  expect(housing_request1.request_status).to eq("COMPLETE")
      end
    end
    describe "ajout de piece invalidé kyc" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
        required_document.reload
        required_document.validation_errors = [{ereeur: "zefsfsfs"}]
        required_document.save
      end
      it "has status kyc en cours" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        # expect(rd.status).to eq("Pièce invalide")
        # expect(housing_request1.reload.housing_request_folder.application_status).to eq("Erreurs de validation dans les pieces fournies")
        # expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Erreurs de validation dans les pieces fournies")
      end
    end
    describe "ajout de piece start treatment" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
        required_document.reload
        
        
        required_document.start_treatment
        required_document.status = "Pièce validée"
        required_document.validity_limit = Date.current + 5.years
        required_document.save
      end
      it "has status kyc en cours" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        expect(rd.status).to eq("Pièce validée")
        expect(housing_request1.reload.housing_request_folder.nud_status).to eq("Dossier validé")
        expect(housing_request1.reload.request_status).to eq("ELIGIBLE")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Dossier validé")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        #  expect(housing_request1.request_status).to eq("COMPLETE")
      end
    end
    describe "ajout de piece invalidé kyc" do
      before :each do
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
        required_document.reload
        required_document.validation_errors = [{ereeur: "zefsfsfs"}]
        required_document.save
      end
      it "has status kyc en cours" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(3)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        # expect(rd.status).to eq("Pièce invalide")
        # expect(housing_request1.reload.housing_request_folder.application_status).to eq("Erreurs de validation dans les pieces fournies")
        # expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Erreurs de validation dans les pieces fournies")
      end
    end
    describe "multi piece" do
      before :each do
        housing_request1.unborn_children = [{
                                              "anticipated_birth" => (Time.current + 10.days).to_s,
                                            }]
        housing_request1.save
        mapp = housing_request1.main_applicant
        mapp.amount_year_minus_two = 2000
        mapp.save
        required_document = housing_request1.reload.housing_request_folder.required_documents[0]
        file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
        blob = file.read
        required_document.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
        required_document.save
      end
      it "has status kyc en cours" do
        expect(housing_request1.reload.housing_request_folder.required_document_ids.length).to eq(4)
        rd = Hermes::RequiredDocument.find(housing_request1.reload.housing_request_folder.required_document_ids[0])
        expect(rd.status).to eq("Prêt pour traitement")
        
        expect(housing_request1.reload.housing_request_folder.nud_status).to eq("Dossier complet prêt pour traitement")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Dossier complet prêt pour traitement")
        housing_request1.reload.housing_request_folder.start_treatment
        expect(housing_request1.reload.housing_request_folder.nud_status).to eq("Dossier validé")
        rd.status = "Pièce validée"
        rd.validity_limit = Date.current + 5.years
        rd.save
        expect(housing_request1.reload.housing_request_folder.nud_status).to eq("Dossier validé")
        expect(housing_request1.reload.housing_request_folder.application_status).to eq("Dossier incomplet")
        expect(housing_request1.reload.housing_request_folder.consultation_status).to eq("Dossier validé")
        expect(housing_request1.request_status).to eq("ELIGIBLE")
      end
    end
  end

  describe 'declared' do 
    let!(:rd) { housing_request1.housing_request_folder.required_documents.first}

    it 'not set by default' do
      rd.reload
      expect(rd.declared).to eq(nil)
      expect(rd.status).to eq('Pièce manquante')
    end

    it 'when true' do
      expect{rd.update(declared: true)}.to change{rd.reload.status}.from('Pièce manquante').to('Pièce validée')
    end

    it 'when false' do
      file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
      blob = file.read
      rd.set_blobs([{filename: "minion.jpg", blob: blob}, {filename: "minion.jpg", blob: blob}])
      rd.save
      expect{rd.update(declared: false)}.to change{rd.reload.status}.from('Prêt pour traitement').to('Pièce manquante')
    end

  end 
end 